@@ -5,6 +5,8 @@ from functools import reduce
 import gtk
 import gettext
 from interface import ParserView
+from tryton.common.focus import (get_invisible_ancestor, find_focused_child,
+    next_focus_widget)
 
 _ = gettext.gettext
 
@@ -16,10 +18,10 @@ class ViewForm(ParserView):
         super(ViewForm, self).__init__(screen, widget, children, state_widgets,
             notebooks, cursor_widget, children_field)
         self.view_type = 'form'
+        self.editable = True
 
-        for widget in self.state_widgets:
-            if isinstance(widget, gtk.Button):
-                widget.connect('clicked', self.button_clicked)
+        for button in self.get_buttons():
+            button.connect('clicked', self.button_clicked)
 
         # Force to display the first time it switches on a page
         # This avoids glitch in position of widgets
@@ -59,17 +61,7 @@ class ViewForm(ParserView):
         for widget_name in self.widgets.keys():
             for widget in self.widgets[widget_name]:
                 widget.destroy()
-            del self.widgets[widget_name]
         self.widget.destroy()
-        self.widget = None
-        self.widgets = None
-        self.screen = None
-        self.state_widgets = None
-
-    def cancel(self):
-        for widgets in self.widgets.itervalues():
-            for widget in widgets:
-                widget.cancel()
 
     def set_value(self, focused_widget=False):
         record = self.screen.current_record
@@ -84,11 +76,7 @@ class ViewForm(ParserView):
                                     and widget.widget.get_focus_child())):
                             widget.set_value(record, field)
 
-    def sel_ids_get(self):
-        if self.screen.current_record:
-            return [self.screen.current_record.id]
-        return []
-
+    @property
     def selected_records(self):
         if self.screen.current_record:
             return [self.screen.current_record]
@@ -99,6 +87,9 @@ class ViewForm(ParserView):
         return any(w.modified for widgets in self.widgets.itervalues()
             for w in widgets)
 
+    def get_buttons(self):
+        return [b for b in self.state_widgets if isinstance(b, gtk.Button)]
+
     def reset(self):
         record = self.screen.current_record
         if record:
@@ -119,6 +110,7 @@ class ViewForm(ParserView):
             fields.sort(key=operator.itemgetter(1), reverse=True)
             for field, _ in fields:
                 record[field].get(record)
+        focused_widget = find_focused_child(self.widget)
         for name, widgets in self.widgets.iteritems():
             field = None
             if record:
@@ -129,19 +121,24 @@ class ViewForm(ParserView):
                 widget.display(record, field)
         for widget in self.state_widgets:
             widget.state_set(record)
+        if focused_widget:
+            invisible_ancestor = get_invisible_ancestor(focused_widget)
+            if invisible_ancestor:
+                new_focused_widget = next_focus_widget(invisible_ancestor)
+                if new_focused_widget:
+                    new_focused_widget.grab_focus()
         return True
 
     def set_cursor(self, new=False, reset_view=True):
-        if reset_view:
-            for notebook in self.notebooks:
-                notebook.set_current_page(0)
+        focus_widget = None
+        if reset_view or not self.widget.has_focus():
+            if reset_view:
+                for notebook in self.notebooks:
+                    notebook.set_current_page(0)
             if self.cursor_widget in self.widgets:
-                self.widgets[self.cursor_widget][0].grab_focus()
-        elif not self.widget.has_focus():
-            self.widgets[self.cursor_widget][0].grab_focus()
+                focus_widget = self.widgets[self.cursor_widget][0]
         record = self.screen.current_record
         position = reduce(lambda x, y: x + len(y), self.widgets, 0)
-        focus_widget = None
         if record:
             for name, widgets in self.widgets.iteritems():
                 for widget in widgets:
@@ -163,9 +160,14 @@ class ViewForm(ParserView):
 
     def button_clicked(self, widget):
         record = self.screen.current_record
+        self.set_value()
         fields = self.get_fields()
         if not record.validate(fields):
-            self.screen.display()
+            self.screen.display(set_cursor=True)
             return
         else:
-            self.screen.button(widget.attrs)
+            widget.handler_block_by_func(self.button_clicked)
+            try:
+                self.screen.button(widget.attrs)
+            finally:
+                widget.handler_unblock_by_func(self.button_clicked)
