@@ -16,6 +16,7 @@ from tryton.common import RPCExecute, RPCException, timezoned_date, \
     datetime_strftime, Tooltips
 from tryton.common.date_widget import DateEntry
 from tryton.common.placeholder_entry import PlaceholderEntry
+from tryton.common.selection import selection_shortcuts
 from tryton.translate import date_format
 
 _ = gettext.gettext
@@ -83,9 +84,10 @@ class DictSelectionEntry(DictEntry):
         widget.child.connect('focus-out-event',
             lambda w, e: self.parent_widget._focus_out())
         widget.child.connect('activate',
-            lambda w, e: self.parent_widget._focus_out())
+            lambda w: self.parent_widget._focus_out())
         widget.connect('notify::active',
             lambda w, e: self.parent_widget._focus_out())
+        selection_shortcuts(widget)
 
         # setting completion and selection
         model = gtk.ListStore(gobject.TYPE_STRING)
@@ -102,14 +104,15 @@ class DictSelectionEntry(DictEntry):
         widget.set_text_column(0)
         widget.child.set_width_chars(width)
         completion = gtk.EntryCompletion()
-        if hasattr(completion, 'set_inline_selection'):
-            completion.set_inline_selection(True)
+        completion.set_inline_selection(True)
         completion.set_model(model)
         widget.child.set_completion(completion)
         completion.set_text_column(0)
         return widget
 
     def get_value(self):
+        if not self.widget.child:  # widget is destroyed
+            return
         text = self.widget.child.get_text()
         value = None
         if text:
