@@ -12,17 +12,11 @@ class Float(Integer):
         super(Float, self).__init__(field_name, model_name, attrs=attrs)
         self.entry.connect('key-press-event', self.key_press_event)
 
-    def set_value(self, record, field):
-        return field.set_client(record, self.entry.get_text())
-
     def display(self, record, field):
         super(Float, self).display(record, field)
-        if not field:
-            self.entry.set_text('')
-            return False
-        digits = field.digits(record)
-        self.entry.set_width_chars(sum(digits))
-        self.entry.set_text(field.get_client(record))
+        if field:
+            digits = field.digits(record, factor=self.factor)
+            self.entry.set_width_chars(sum(digits))
 
     def key_press_event(self, widget, event):
         for name in ('KP_Decimal', 'KP_Separator'):
@@ -43,7 +37,7 @@ class Float(Integer):
         if new_value in ('-', decimal_point):
             return
 
-        digits = self.field.digits(self.record)
+        digits = self.field.digits(self.record, factor=self.factor)
 
         try:
             locale.atof(new_value)
