@@ -196,8 +196,10 @@ class ParserForm(ParserInterface):
                 continue
             attrs = common.node_attributes(node)
             if not cursor_widget:
-                if attrs.get('name') and fields.get(attrs['name']) \
-                        and attrs['name'] != self.screen.exclude_field:
+                if (attrs.get('name') and fields.get(attrs['name'])
+                        and not int(attrs.get('invisible', 0))
+                        and not int(attrs.get('readonly', 0))
+                        and attrs['name'] != self.screen.exclude_field):
                     cursor_widget = attrs.get('name')
             yexpand = int(attrs.get('yexpand', 0))
             yfill = int(attrs.get('yfill', 0))
@@ -457,6 +459,7 @@ class ParserForm(ParserInterface):
                     text = fields[attrs['name']].attrs['string']
                 if attrs.get('string'):
                     text = attrs['string']
+                widget.set_homogeneous(bool(int(attrs.get('homogeneous', 0))))
 
                 frame = Frame(text, attrs=attrs, widgets=widgets)
                 frame.add(widget)
@@ -469,13 +472,13 @@ class ParserForm(ParserInterface):
                 hpaned = gtk.HPaned()
                 container.wid_add(hpaned, colspan=int(attrs.get('colspan', 4)),
                         yexpand=True, yfill=True)
-                (widget, widgets, state_widgets, spam, notebook_list2,
+                (widget, widgets, state_widgets2, spam, notebook_list2,
                     cursor_widget2) = self.parse(model_name, node, fields,
                         paned=hpaned, tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                state_widgets += state_widgets
+                state_widgets += state_widgets2
                 for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
@@ -485,7 +488,7 @@ class ParserForm(ParserInterface):
                 vpaned = gtk.VPaned()
                 container.wid_add(vpaned, colspan=int(attrs.get('colspan', 4)),
                         yexpand=True, yfill=True)
-                (widget, widgets, state_widgets2, spam, notebook_list,
+                (widget, widgets, state_widgets2, spam, notebook_list2,
                     cursor_widget2) = self.parse(model_name, node, fields,
                         paned=vpaned, tooltips=tooltips)
                 if not cursor_widget:
@@ -519,7 +522,7 @@ from calendar import Calendar, DateTime, Time
 from float import Float
 from integer import Integer
 from selection import Selection
-from char import Char, Sha
+from char import Char, Password
 from float_time import FloatTime
 from checkbox import CheckBox
 from reference import Reference
@@ -534,6 +537,7 @@ from progressbar import ProgressBar
 from one2one import One2One
 from richtextbox import RichTextBox
 from dictionary import DictWidget
+from multiselection import MultiSelection
 
 
 WIDGETS_TYPE = {
@@ -546,7 +550,7 @@ WIDGETS_TYPE = {
     'biginteger': (Integer, 1, False, False),
     'selection': (Selection, 1, False, False),
     'char': (Char, 1, False, False),
-    'sha': (Sha, 1, False, False),
+    'password': (Password, 1, False, False),
     'float_time': (FloatTime, 1, False, False),
     'boolean': (CheckBox, 1, False, False),
     'reference': (Reference, 1, False, False),
@@ -564,4 +568,5 @@ WIDGETS_TYPE = {
     'one2one': (One2One, 1, False, False),
     'richtext': (RichTextBox, 1, True, True),
     'dict': (DictWidget, 1, False, False),
+    'multiselection': (MultiSelection, 1, True, True),
 }
