@@ -30,10 +30,18 @@ class ParserView(object):
         self.notebooks = notebooks
         self.cursor_widget = cursor_widget
         self.children_field = children_field
+        self.view_id = None
+
+    @property
+    def selected_records(self):
+        return []
 
     def get_fields(self):
         return self.children.keys()
 
+    def get_buttons(self):
+        return []
+
     @property
     def modified(self):
         return False
