@@ -18,10 +18,10 @@ class TreeView(gtk.TreeView):
         super(TreeView, self).__init__()
         self.cells = {}
 
-    def next_column(self, path, column=None, _sign=1):
+    def next_column(self, path, column=None, editable=True, _sign=1):
         columns = self.get_columns()
         if column is None:
-            column = columns[-1 * _sign]
+            column = columns[-1 if _sign > 0 else 0]
         model = self.get_model()
         record = model.get_value(model.get_iter(path), 0)
         if _sign < 0:
@@ -34,13 +34,17 @@ class TreeView(gtk.TreeView):
             field = record[column.name]
             field.state_set(record, states=('readonly', 'invisible'))
             invisible = field.get_state_attrs(record).get('invisible', False)
-            readonly = field.get_state_attrs(record).get('readonly', False)
+            if editable:
+                readonly = field.get_state_attrs(record).get('readonly', False)
+            else:
+                readonly = False
             if not (invisible or readonly):
                 break
         return column
 
-    def prev_column(self, path, column=None):
-        return self.next_column(path, column=column, _sign=-1)
+    def prev_column(self, path, column=None, editable=True):
+        return self.next_column(path, column=column, editable=editable,
+            _sign=-1)
 
 
 class EditableTreeView(TreeView):
@@ -55,8 +59,6 @@ class EditableTreeView(TreeView):
 
     def on_quit_cell(self, current_record, fieldname, value, callback=None):
         field = current_record[fieldname]
-        if hasattr(field, 'editabletree_entry'):
-            del field.editabletree_entry
         cell = self.cells[fieldname]
 
         # The value has not changed and is valid ... do nothing.
@@ -102,6 +104,7 @@ class EditableTreeView(TreeView):
         self.grab_focus()
         if focus_column and (focus_column._type in ('boolean')):
             start_editing = False
+        self.scroll_to_cell(path, focus_column, use_align=False)
         super(EditableTreeView, self).set_cursor(path, focus_column,
                 start_editing)
 
@@ -225,6 +228,11 @@ class EditableTreeView(TreeView):
             # store in the record the entry widget to get the value in
             # set_value
             field.editabletree_entry = entry
+
+            def remove_widget(cell):
+                if hasattr(field, 'editabletree_entry'):
+                    del field.editabletree_entry
+            entry.connect('remove-widget', remove_widget)
             record.modified_fields.setdefault(column.name)
             return False
 
