@@ -38,18 +38,25 @@ class WinForm(NoModal):
         self.win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.win.set_icon(TRYTON_ICON)
         self.win.set_has_separator(False)
-        if hasattr(self.win, 'set_deletable'):
-            self.win.set_deletable(False)
+        self.win.set_deletable(False)
         self.win.connect('close', self.close)
         self.win.connect('response', self.response)
 
         self.accel_group = gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
 
+        readonly = self.screen.readonly or self.screen.group.readonly
+
         self.but_ok = None
         self.but_new = None
-        self.but_cancel = self.win.add_button(gtk.STOCK_CANCEL,
-            gtk.RESPONSE_CANCEL)
+
+        if view_type == 'form':
+            if not new and self.screen.current_record.id < 0:
+                stock_id = gtk.STOCK_DELETE
+            else:
+                stock_id = gtk.STOCK_CANCEL
+            self.but_cancel = self.win.add_button(stock_id,
+                gtk.RESPONSE_CANCEL)
 
         if new and self.many:
             self.but_new = self.win.add_button(gtk.STOCK_NEW,
@@ -79,6 +86,7 @@ class WinForm(NoModal):
         title.set_label(self.screen.current_view.title)
         title.set_padding(20, 3)
         title.set_alignment(0.0, 0.5)
+        title.set_size_request(0, -1)  # Allow overflow
         title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
         title.show()
 
@@ -134,7 +142,7 @@ class WinForm(NoModal):
                 self.but_add.add(img_add)
                 self.but_add.set_relief(gtk.RELIEF_NONE)
                 hbox.pack_start(self.but_add, expand=False, fill=False)
-                if not access['read']:
+                if not access['read'] or readonly:
                     self.but_add.set_sensitive(False)
 
                 self.but_remove = gtk.Button()
@@ -147,7 +155,7 @@ class WinForm(NoModal):
                 self.but_remove.add(img_remove)
                 self.but_remove.set_relief(gtk.RELIEF_NONE)
                 hbox.pack_start(self.but_remove, expand=False, fill=False)
-                if not access['read']:
+                if not access['read'] or readonly:
                     self.but_remove.set_sensitive(False)
 
                 hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
@@ -161,7 +169,7 @@ class WinForm(NoModal):
             self.but_new.add(img_new)
             self.but_new.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_new, expand=False, fill=False)
-            if not access['create']:
+            if not access['create'] or readonly:
                 self.but_new.set_sensitive(False)
 
             self.but_del = gtk.Button()
@@ -174,7 +182,7 @@ class WinForm(NoModal):
             self.but_del.add(img_del)
             self.but_del.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_del, expand=False, fill=False)
-            if not access['delete']:
+            if not access['delete'] or readonly:
                 self.but_del.set_sensitive(False)
 
             self.but_undel = gtk.Button()
@@ -188,6 +196,8 @@ class WinForm(NoModal):
             self.but_undel.add(img_undel)
             self.but_undel.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_undel, expand=False, fill=False)
+            if not access['delete'] or readonly:
+                self.but_undel.set_sensitive(False)
 
             hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
 
@@ -257,7 +267,7 @@ class WinForm(NoModal):
             self.screen.screen_container.alternate_viewport.connect(
                     'key-press-event', self.on_keypress)
 
-        if self.save_current:
+        if self.save_current and not new:
             self.screen.signal_connect(self, 'record-message',
                 self.activate_save)
             self.screen.signal_connect(self, 'record-modified',
@@ -349,9 +359,10 @@ class WinForm(NoModal):
 
     def _sig_label(self, screen, signal_data):
         name = '_'
+        access = common.MODELACCESS[screen.model_name]
+        readonly = screen.group.readonly
         if signal_data[0] >= 1:
             name = str(signal_data[0])
-            self.but_del.set_sensitive(True)
             if self.domain is not None:
                 self.but_remove.set_sensitive(True)
             if signal_data[0] < signal_data[1]:
@@ -362,8 +373,9 @@ class WinForm(NoModal):
                 self.but_pre.set_sensitive(True)
             else:
                 self.but_pre.set_sensitive(False)
-            self.but_del.set_sensitive(True)
-            self.but_undel.set_sensitive(True)
+            if access['delete'] and not readonly:
+                self.but_del.set_sensitive(True)
+                self.but_undel.set_sensitive(True)
         else:
             self.but_del.set_sensitive(False)
             self.but_undel.set_sensitive(False)
@@ -389,9 +401,9 @@ class WinForm(NoModal):
         validate = False
         cancel_responses = (gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT)
         self.screen.current_view.set_value()
-        if not self.win.get_focus():
-            return
+        readonly = self.screen.group.readonly
         if (response_id not in cancel_responses
+                and not readonly
                 and self.screen.current_record is not None):
             validate = self.screen.current_record.validate(
                 self.screen.current_view.get_fields())
@@ -417,6 +429,7 @@ class WinForm(NoModal):
                 self.new()
                 return
         if (self.screen.current_record
+                and not readonly
                 and response_id in cancel_responses):
             if (self.screen.current_record.id < 0
                     or self.save_current):
