@@ -15,7 +15,7 @@ _ = gettext.gettext
 class WinImport(NoModal):
     "Window import"
 
-    def __init__(self, model):
+    def __init__(self, model, context):
         super(WinImport, self).__init__()
         self.dialog = gtk.Dialog(title=_("Import from CSV"),
             parent=self.parent, flags=gtk.DIALOG_DESTROY_WITH_PARENT)
@@ -49,7 +49,7 @@ class WinImport(NoModal):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-list-add', gtk.ICON_SIZE_BUTTON)
         button_add.set_image(img_button)
-        button_add.connect_after('clicked',  self.sig_sel)
+        button_add.connect_after('clicked', self.sig_sel)
         vbox_buttons.pack_start(button_add, False, False, 0)
 
         button_remove = gtk.Button(_("_Remove"), stock=None,
@@ -58,7 +58,7 @@ class WinImport(NoModal):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-list-remove', gtk.ICON_SIZE_BUTTON)
         button_remove.set_image(img_button)
-        button_remove.connect_after('clicked',  self.sig_unsel)
+        button_remove.connect_after('clicked', self.sig_unsel)
         vbox_buttons.pack_start(button_remove, False, False, 0)
 
         button_remove_all = gtk.Button(_("Clear"), stock=None,
@@ -67,7 +67,7 @@ class WinImport(NoModal):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-clear', gtk.ICON_SIZE_BUTTON)
         button_remove_all.set_image(img_button)
-        button_remove_all.connect_after('clicked',  self.sig_unsel_all)
+        button_remove_all.connect_after('clicked', self.sig_unsel_all)
         vbox_buttons.pack_start(button_remove_all, False, False, 0)
 
         hseparator_buttons = gtk.HSeparator()
@@ -79,7 +79,7 @@ class WinImport(NoModal):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-find', gtk.ICON_SIZE_BUTTON)
         button_autodetect.set_image(img_button)
-        button_autodetect.connect_after('clicked',  self.sig_autodetect)
+        button_autodetect.connect_after('clicked', self.sig_autodetect)
         vbox_buttons.pack_start(button_autodetect, False, False, 0)
 
         frame_import = gtk.Frame()
@@ -166,6 +166,7 @@ class WinImport(NoModal):
         self.dialog.vbox.pack_start(dialog_vbox)
 
         self.model = model
+        self.context = context
         self.fields_data = {}
 
         self.import_csv_file.set_current_folder(CONFIG['client.default_path'])
@@ -230,12 +231,16 @@ class WinImport(NoModal):
 
     def _get_fields(self, model):
         try:
-            return RPCExecute('model', model, 'fields_get', None)
+            return RPCExecute('model', model, 'fields_get', None,
+                context=self.context)
         except RPCException:
             return ''
 
     def on_row_expanded(self, treeview, iter, path):
         child = self.model1.iter_children(iter)
+        # autodetect could call for node without children
+        if child is None:
+            return
         if self.model1.get_value(child, 0) is None:
             prefix_field = self.model1.get_value(iter, 1)
             name, model = self.fields[prefix_field]
@@ -248,10 +253,9 @@ class WinImport(NoModal):
         if not fname:
             common.message(_('You must select an import file first!'))
             return True
-        csvsep = self.import_csv_sep.get_text()
-        csvdel = self.import_csv_del.get_text()
-        csvcode = self.import_csv_enc.get_active_text() \
-                or 'UTF-8'
+        csvsep = self.import_csv_sep.get_text() or None
+        csvdel = self.import_csv_del.get_text() or None
+        csvcode = self.import_csv_enc.get_active_text() or 'UTF-8'
 
         self.import_csv_skip.set_value(1)
         try:
@@ -338,11 +342,9 @@ class WinImport(NoModal):
             }
             self.destroy()
             if csv_data['fname']:
-                return self.import_csv(csv_data, fields, self.model)
-            return False
+                self.import_csv(csv_data, fields, self.model)
         else:
             self.destroy()
-            return False
 
     def import_csv(self, csv_data, fields, model):
         # TODO: make it works with references
@@ -354,23 +356,14 @@ class WinImport(NoModal):
         for line in data:
             if not line:
                 continue
-            datas.append([x.decode(csv_data['combo']).encode('utf-8') \
+            datas.append([x.decode(csv_data['combo']).encode('utf-8')
                     for x in line])
         try:
-            res = RPCExecute('model', model, 'import_data', fields, datas)
+            count = RPCExecute('model', model, 'import_data', fields, datas,
+                context=self.context)
         except RPCException:
-            return False
-        if res[0] >= 0:
-            if res[0] == 1:
-                common.message(_('%d record imported!') % res[0])
-            else:
-                common.message(_('%d records imported!') % res[0])
+            return
+        if count == 1:
+            common.message(_('%d record imported!') % count)
         else:
-            buf = ''
-            for key, val in res[1].items():
-                buf += ('\t%s: %s\n' % (str(key), str(val)))
-            common.error(_('Importation Error!'),
-                _('Error importing record %(record)s\n'
-                    '%(error_title)s\n\n%(traceback)s') %
-                {'record': buf, 'error_title': res[2], 'traceback': res[3]})
-        return True
+            common.message(_('%d records imported!') % count)
