@@ -3,17 +3,16 @@
 import gtk
 import pango
 import gettext
-import socket
 
 from tryton.signal_event import SignalEvent
-import tryton.rpc as rpc
 import tryton.common as common
 from tryton.gui.window.view_form.screen import Screen
 from tryton.gui import Main
-from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
+from tryton.exceptions import TrytonServerError
 from tryton.gui.window.nomodal import NoModal
 from tryton.common.button import Button
-from tryton.common import RPCExecute, RPCException, TRYTON_ICON
+from tryton.common import RPCExecute, RPCException
+from tryton.common import TRYTON_ICON
 _ = gettext.gettext
 
 
@@ -28,6 +27,7 @@ class Wizard(object):
         self.id = None
         self.ids = None
         self.action = None
+        self.action_id = None
         self.direct_print = False
         self.email_print = False
         self.email = False
@@ -46,6 +46,7 @@ class Wizard(object):
     def run(self, action, data, direct_print=False, email_print=False,
             email=None, context=None):
         self.action = action
+        self.action_id = data.get('action_id')
         self.id = data.get('id')
         self.ids = data.get('ids')
         self.model = data.get('model')
@@ -53,84 +54,77 @@ class Wizard(object):
         self.email_print = email_print
         self.email = email
         self.context = context
-        try:
-            result = RPCExecute('wizard', action, 'create')
-        except RPCException:
-            return
-        self.session_id, self.start_state, self.end_state = result
-        self.state = self.start_state
-        self.process()
+
+        def callback(result):
+            try:
+                result = result()
+            except RPCException:
+                return
+            self.session_id, self.start_state, self.end_state = result
+            self.state = self.start_state
+            self.process()
+        RPCExecute('wizard', action, 'create', callback=callback)
 
     def process(self):
         from tryton.action import Action
         if self.__processing or self.__waiting_response:
             return
-        try:
-            self.__processing = True
-            while self.state != self.end_state:
-                ctx = self.context.copy()
-                ctx['active_id'] = self.id
-                ctx['active_ids'] = self.ids
-                ctx['active_model'] = self.model
-                if self.screen:
-                    data = {
-                        self.screen_state: self.screen.get_on_change_value(),
-                        }
-                else:
-                    data = {}
-                try:
-                    result = RPCExecute('wizard', self.action, 'execute',
-                        self.session_id, data, self.state, context=ctx)
-                except RPCException, rpc_exception:
-                    if (not isinstance(rpc_exception.exception,
-                            TrytonServerError)
-                            or not self.screen):
-                        self.state = self.end_state
-                    break
+        self.__processing = True
 
-                if 'view' in result:
-                    self.clean()
-                    view = result['view']
-                    self.update(view['fields_view'], view['defaults'],
-                        view['buttons'])
-                    self.screen_state = view['state']
-                    self.__waiting_response = True
-                else:
+        ctx = self.context.copy()
+        ctx['active_id'] = self.id
+        ctx['active_ids'] = self.ids
+        ctx['active_model'] = self.model
+        ctx['action_id'] = self.action_id
+        if self.screen:
+            data = {
+                self.screen_state: self.screen.get_on_change_value(),
+                }
+        else:
+            data = {}
+
+        def callback(result):
+            try:
+                result = result()
+            except RPCException, rpc_exception:
+                if (not isinstance(rpc_exception.exception,
+                        TrytonServerError)
+                        or not self.screen):
                     self.state = self.end_state
+                    self.end()
+                self.__processing = False
+                return
 
-                if 'actions' in result:
-                    sensitive_widget = self.widget.get_toplevel()
-                    if not 'view' in result:
-                        self.end()
-                    for action in result['actions']:
-                        Action._exec_action(*action, context=ctx)
-                    if (not 'view' in result
-                            or (action[0]['type'] == 'ir.action.wizard'
-                                and not sensitive_widget.props.sensitive)):
-                        return
-                if self.__waiting_response:
-                    break
+            if 'view' in result:
+                self.clean()
+                view = result['view']
+                self.update(view['fields_view'], view['defaults'],
+                    view['buttons'])
+                self.screen_state = view['state']
+                self.__waiting_response = True
+            else:
+                self.state = self.end_state
+
+            def execute_actions():
+                for action in result.get('actions', []):
+                    Action._exec_action(*action, context=self.context.copy())
 
             if self.state == self.end_state:
-                self.end()
-        finally:
+                self.end(lambda *a: execute_actions())
+            else:
+                execute_actions()
             self.__processing = False
 
+        RPCExecute('wizard', self.action, 'execute', self.session_id, data,
+            self.state, context=ctx, callback=callback)
+
     def destroy(self):
         if self.screen:
             self.screen.destroy()
-            del self.screen
-        del self.widget
 
-    def end(self):
-        try:
-            RPCExecute('wizard', self.action, 'delete', self.session_id,
-                process_exception=False)
-            if self.action == 'ir.module.module.config_wizard':
-                rpc.context_reload()
-                Main.get_main().sig_win_menu()
-        except (TrytonServerError, socket.error, TrytonServerUnavailable):
-            pass
+    def end(self, callback=None):
+        RPCExecute('wizard', self.action, 'delete', self.session_id,
+            process_exception=False, callback=callback)
 
     def clean(self):
         for widget in self.widget.get_children():
@@ -143,7 +137,7 @@ class Wizard(object):
         self.screen.current_view.set_value()
         if (not self.screen.current_record.validate()
                 and state != self.end_state):
-            self.screen.display()
+            self.screen.display(set_cursor=True)
             return
         self.state = state
         self.process()
@@ -166,7 +160,8 @@ class Wizard(object):
             self._get_button(button)
 
         self.screen = Screen(view['model'], mode=[], context=self.context)
-        self.screen.add_view(view, display=True)
+        self.screen.add_view(view)
+        self.screen.switch_view()
         self.screen.widget.show()
         self.screen.signal_connect(self, 'record-modified',
             self._record_modified)
@@ -176,6 +171,7 @@ class Wizard(object):
         title.set_label(self.screen.current_view.title)
         title.set_padding(20, 4)
         title.set_alignment(0.0, 0.5)
+        title.set_size_request(0, -1)  # Allow overflow
         title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
         title.show()
 
@@ -226,7 +222,7 @@ class Wizard(object):
         self.widget.pack_start(self.scrolledwindow)
 
         self.screen.new(default=False)
-        self.screen.current_record.set_default(defaults, modified=True)
+        self.screen.current_record.set_default(defaults)
         self.screen.set_cursor()
 
 
@@ -274,8 +270,8 @@ class WizardForm(Wizard, SignalEvent):
             self.toolbar_box.remove(toolbar)
         super(WizardForm, self).destroy()
 
-    def end(self):
-        super(WizardForm, self).end()
+    def end(self, callback=None):
+        super(WizardForm, self).end(callback=callback)
         Main.get_main()._win_del(self.widget)
 
     def set_cursor(self):
@@ -294,11 +290,9 @@ class WizardDialog(Wizard, NoModal):
             gtk.DIALOG_DESTROY_WITH_PARENT)
         self.dia.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.dia.set_icon(TRYTON_ICON)
-        if hasattr(self.dia, 'set_deletable'):
-            self.dia.set_deletable(False)
+        self.dia.set_deletable(False)
         self.dia.connect('close', self.close)
         self.dia.connect('response', self.response)
-        self.dia.connect('state-changed', self.state_changed)
 
         self.accel_group = gtk.AccelGroup()
         self.dia.add_accel_group(self.accel_group)
@@ -333,7 +327,7 @@ class WizardDialog(Wizard, NoModal):
         self.dia.show()
         common.center_window(self.dia, self.parent, self.sensible_widget)
 
-    def destroy(self):
+    def destroy(self, action=None):
         super(WizardDialog, self).destroy()
         self.dia.destroy()
         NoModal.destroy(self)
@@ -347,18 +341,29 @@ class WizardDialog(Wizard, NoModal):
             dialog = self.page.dialogs[-1]
         else:
             dialog = self.page
-        if (hasattr(dialog, 'screen')
+        if (getattr(dialog, 'screen', None)
                 and dialog.screen.current_record
-                and self.sensible_widget != main.window
-                and self.ids):
-            dialog.screen.reload(self.ids, written=True)
+                and self.sensible_widget != main.window):
+            if dialog.screen.model_name == self.model:
+                ids = self.ids
+            else:
+                # Wizard run from a children record so reload parent record
+                ids = [dialog.screen.current_record.id]
+            dialog.screen.reload(ids, written=True)
+            if action:
+                dialog.screen.client_action(action)
 
-    def end(self):
-        super(WizardDialog, self).end()
-        self.destroy()
+    def end(self, callback=None):
+        def end_callback(action):
+            self.destroy(action=action())
+            if callback:
+                callback()
+        super(WizardDialog, self).end(callback=end_callback)
 
     def close(self, widget, event=None):
         widget.emit_stop_by_name('close')
+        if self.end_state in self.states:
+            self.states[self.end_state].clicked()
         return True
 
     def show(self):
@@ -366,7 +371,3 @@ class WizardDialog(Wizard, NoModal):
 
     def hide(self):
         self.dia.hide()
-
-    def state_changed(self, widget, state):
-        if self.dia.props.sensitive and state == gtk.STATE_INSENSITIVE:
-            self.process()
